/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.monitor;

import java.util.Timer;
import java.util.TimerTask;

import javax.management.MalformedObjectNameException;

import org.apache.log4j.Appender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.ConsoleAppender;

import org.jboss.logging.appender.DailyRollingFileAppender;

import org.jboss.system.ServiceMBeanSupport;

import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * This class implements the LoggingMonitor service which provides the ability
 * to create monitoring logs for various MBeans and their attributes.
 */
public class LoggingMonitor extends ServiceMBeanSupport
                            implements LoggingMonitorMBean
{
    private String           _filename;
    private boolean          _appendToFile;
    private MonitoredMBean[] _monitoredObjects;
    private long             _monitorPeriod;

    private Appender         _appender;
    private Timer            _timer;

    /**
     * Default constructor.
     */
    public LoggingMonitor()
    {
        _appendToFile = true;
    }

    /*
     * LoggingMonitorMBean implementation.
     */

    public String getFilename()
    {
        return _filename;
    }

    public void setFilename(String filename)
    {
        if (filename == null || filename.length() <= 0)
        {
            final String MESSAGE =
                "Logging monitor's filename can not be null or empty.";

            throw new IllegalArgumentException(MESSAGE);
        }

        _filename = filename;
    }

    public boolean getAppendToFile()
    {
        return _appendToFile;
    }

    public void setAppendToFile(boolean appendToFile)
    {
        _appendToFile = appendToFile;
    }

    public void setMonitoredObjects(Element monitoredObjects)
    throws MalformedObjectNameException
    {
        final String MONITORED_MBEAN_ELEMENT = "monitoredmbean";

        NodeList monitoredMBeans =
            monitoredObjects.getElementsByTagName(MONITORED_MBEAN_ELEMENT);

        int mbeanCount = monitoredMBeans.getLength();

        if (mbeanCount < 1)
        {
            throw createMissingElementException(MONITORED_MBEAN_ELEMENT);
        }

        _monitoredObjects = new MonitoredMBean[mbeanCount];

        for (int i = 0; i < mbeanCount; ++i)
        {
            Node monitoredMBean = monitoredMBeans.item(i);
            _monitoredObjects[i] = toMonitoredMBean((Element) monitoredMBean);
        }
    }

    public long getMonitorPeriod()
    {
        return _monitorPeriod;
    }

    public void setMonitorPeriod(long monitorPeriod)
    {
        final String MESSAGE =
            "Logging monitor's monitor period must be a positive, non-zero value";

        if (monitorPeriod < 1)
        {
            throw new IllegalArgumentException(MESSAGE);
        }

        _monitorPeriod = monitorPeriod;
    }

    /*
     * ServiceMBeanSupport overrides.
     */

    protected void startService()
    {
        DailyRollingFileAppender appender = new DailyRollingFileAppender();
        appender.setFile(_filename);
        appender.setAppend(_appendToFile);
        appender.setDatePattern("\'.\'yyyy-MM-dd");
        appender.setLayout(new PatternLayout("%d %-5p [%c{1}] %m%n"));
        appender.setThreshold(Level.INFO);
        appender.activateOptions();
        _appender = appender;

        for (int i = 0; i < _monitoredObjects.length; ++i)
        {
            _monitoredObjects[i].getLogger().addAppender(appender);
        }

        TimerTask task =
            new LoggingMonitorTimerTask(_monitoredObjects);

        _timer = new Timer();
        _timer.schedule(task, 0, _monitorPeriod);

        if (log.isDebugEnabled())
        {
            log.debug("Logging monitor started logging to " + _filename);
        }
    }

    protected void stopService()
    {
        _timer.cancel();

        for (int i = 0; i < _monitoredObjects.length; ++i)
        {
            _monitoredObjects[i].getLogger().removeAllAppenders();
        }

        _appender.close();

        if (log.isDebugEnabled())
        {
            log.debug("Logging monitor stopped logging to " + _filename);
        }
    }

    /**
     * Retrieves the attributes of the MBean to monitor.
     *
     * @param monitoredMBean a MBean, represented as a XML DOM element, for
     *                       which to retrieve the attributes to monitor.
     *
     * @return the attributes of the MBean to monitor.
     */
    private String[] getMonitoredAttributes(Element monitoredMBean)
    {
        final String ATTRIBUTE_ELEMENT = "attribute";

        NodeList monitoredAttributes =
            monitoredMBean.getElementsByTagName(ATTRIBUTE_ELEMENT);

        int monitoredAttributesCount = monitoredAttributes.getLength();

        if (monitoredAttributesCount < 1)
        {
            throw createMissingElementException(ATTRIBUTE_ELEMENT);
        }

        String[] attributes = new String[monitoredAttributesCount];

        for (int i = 0; i < monitoredAttributesCount; ++i)
        {
            Node node = monitoredAttributes.item(i);
            Node attribute = node.getFirstChild();

            if (attribute.getNodeType() != Node.TEXT_NODE)
            {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR,
                                       "Unexpected node type inside <attribute> for monitored MBean.");
            }

            attributes[i] = ((Text) attribute).getData();
        }

        return attributes;
    }

    /**
     * Converts the specified XML DOM element to a monitored MBean.
     *
     * @param element the XML DOM element to be converted.
     *
     * @return a monitored MBean represented by the specified XML DOM element.
     *
     * @throws MalformedObjectNameException if the specified XML DOM element
     *                                      does not contain a valid object
     *                                      name.
     */
    private MonitoredMBean toMonitoredMBean(Element element)
    throws MalformedObjectNameException
    {
        final String NAME_ATTRIBUTE = "name";
        String objectName = element.getAttribute("name");

        if (objectName == "")
        {
            throw createAttributeNotFoundException(NAME_ATTRIBUTE);
        }

        final String LOGGER_ATTRIBUTE = "logger";
        String loggerName = element.getAttribute("logger");

        if (loggerName == "")
        {
            throw createAttributeNotFoundException(LOGGER_ATTRIBUTE);
        }

        Logger logger = Logger.getLogger("monitor." + loggerName.toLowerCase());
        logger.setAdditivity(false);
        logger.setLevel(Level.INFO);

        String[] attributes = getMonitoredAttributes(element);

        return new MonitoredMBean(objectName, attributes, logger);
    }

    /**
     * Creates a <code>DOMException</code> relating that the specified attribute
     * was not found as expected.
     *
     * @param attribute the expected attribute.
     *
     * @return a <code>DOMException</code> relating that the specified attribute
     *         was not found as expected.
     */
    private DOMException createAttributeNotFoundException(String attribute)
    {
        final String MESSAGE_PREFIX = "Missing expected ";

        final String MESSAGE_SUFFIX =
            " attribute of a <monitoredmbean> element.";

        final char QUOTE = '\"';

        StringBuffer message = new StringBuffer(MESSAGE_PREFIX);
        message.append(QUOTE);
        message.append(attribute);
        message.append(QUOTE);
        message.append(MESSAGE_SUFFIX);

        return new DOMException(DOMException.NOT_FOUND_ERR, message.toString());
    }

    /**
     * Creates a <code>DOMException</code> relating that at least one occurrence
     * of the specified element was not found as expected.
     *
     * @param element the expected element.
     *
     * @return a <code>DOMException</code> relating that at least one occurrence
     *         of the specified element was not found as expected.
     */
    private DOMException createMissingElementException(String element)
    {
        final String MESSAGE_PREFIX = "At least one <";
        final String MESSAGE_SUFFIX = "> element is expected.";

        return new DOMException(DOMException.NOT_FOUND_ERR,
                                MESSAGE_PREFIX + element + MESSAGE_SUFFIX);
    }
}
