/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.monitor;

import javax.management.JMException;
import javax.management.MalformedObjectNameException;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.apache.log4j.Logger;

import org.jboss.mx.util.MBeanServerLocator;

/**
 * This class encapsulates the information necessary for monitoring an MBean.
 */
class MonitoredMBean
{
    private ObjectName  _objectName;
    private String[]    _attributes;
    private Logger      _logger;

    private MBeanServer _mbeanServer;

    /**
     * Constructor.
     *
     * @param objectName this monitored MBean's object name.
     *
     * @param attributes the attributes of this MBean to monitor.
     *
     * @param logger the logger for this monitored MBean.
     *
     * @param throws MalformedObjectNameException if the specified object name
     *                                            is invalid.
     */
    public MonitoredMBean(String objectName, String[] attributes, Logger logger)
    throws MalformedObjectNameException
    {
        _objectName = new ObjectName(objectName);
        _attributes = attributes;
        _logger = logger;
        _mbeanServer = MBeanServerLocator.locateJBoss();
    }

    /**
     * Retrieves the object name of this monitored MBean.
     *
     * @return this monitored MBean's object name.
     */
    public ObjectName getObjectName()
    {
        return _objectName;
    }

    /**
     * Retrieves the attributes of this monitored MBean.
     *
     * @return this monitored MBean's attributes.
     */
    public String[] getAttributes()
    {
        return _attributes;
    }

    /**
     * Retrieves the logger for this monitored MBean.
     *
     * @return this monitored MBean's logger.
     */
    public Logger getLogger()
    {
        return _logger;
    }

    /**
     * Logs the format message for this monitored MBean.
     */
    public void logFormat()
    {
        _logger.info(buildFormatMessage());
    }

    /**
     * Logs the attributes of this MBean that are being monitored.
     *
     * @throws JMException if the retrieval of a MBean attribute causes such an
     *                     exception.
     */
    public void logAttributes()
    throws JMException
    {
        StringBuffer message = new StringBuffer();

        for (int j = 0; j < _attributes.length; ++j)
        {
            Object attributeValue =
                _mbeanServer.getAttribute(_objectName, _attributes[j]);

            message.append(attributeValue);

            if (j < (_attributes.length - 1))
            {
                message.append(",");
            }
        }

        _logger.info(message);
    }

    /**
     * Builds the format message for this monitored MBean.
     *
     * @return this monitored MBean's format message.
     */
    private String buildFormatMessage()
    {
        StringBuffer message =
            new StringBuffer(_objectName.toString());

        message.append(" monitor format: (");

        for (int i = 0; i < _attributes.length; ++i)
        {
            message.append(_attributes[i]);

            if (i < (_attributes.length - 1))
            {
                message.append(',');
            }
        }

        message.append(")");

        return message.toString();
    }
}

